# Examples

In this tab, we will provide a list of example code to use the {{our}} model to address 
several different deep function learning tasks with the `tinybig` library.

We assume you have read the PRN paper, as well as the provided tutorials already.
Therefore, we will not explain the technical details for the model components in the example anymore.

## Examples of {{our}} for Continuous Function Approximation

{% set elementary_example_files %}<a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/elementary_example.ipynb"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/ipynb_icon.png" alt="Jupyter Logo" style="height: 2em; vertical-align: middle; margin-right: 10px;"></a> <a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/configs/elementary_configs.yaml"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/yaml_icon.png" alt="Yaml Logo" style="height: 2em; vertical-align: middle; margin-right: 4px;"></a> <a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/elementary_example.py"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/python_icon.svg" alt="Python Logo" style="height: 2em; vertical-align: middle; margin-right: 10px;"></a>{% endset %}
{% set composite_example_files %}<a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/composite_example.ipynb"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/ipynb_icon.png" alt="Jupyter Logo" style="height: 2em; vertical-align: middle; margin-right: 10px;"></a> <a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/configs/composite_configs.yaml"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/yaml_icon.png" alt="Yaml Logo" style="height: 2em; vertical-align: middle; margin-right: 4px;"></a> <a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/composite_example.py"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/python_icon.svg" alt="Python Logo" style="height: 2em; vertical-align: middle; margin-right: 10px;"></a>{% endset %}
{% set feynman_example_files %}<a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/feynman_example.ipynb"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/ipynb_icon.png" alt="Jupyter Logo" style="height: 2em; vertical-align: middle; margin-right: 10px;"></a> <a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/configs/feynman_configs.yaml"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/yaml_icon.png" alt="Yaml Logo" style="height: 2em; vertical-align: middle; margin-right: 4px;"></a> <a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/feynman_example.py"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/python_icon.svg" alt="Python Logo" style="height: 2em; vertical-align: middle; margin-right: 10px;"></a>{% endset %}


|              Example ID               |           Example Title           | Released Date |         Related Files         |
|:-------------------------------------:|:---------------------------------:|:-------------:|:-----------------------------:|
| [Example 1](./function/elementary.md) | Elementary Function Approximation | July 7, 2024  | {{elementary_example_files}}  |
| [Example 2](./function/composite.md)  | Composite Function Approximation  | July 8, 2024  |  {{composite_example_files}}  |
|  [Example 3](./function/feynman.md)   |  Feynman Function Approximation   | July 8, 2024  |   {{feynman_example_files}}   |

<!-- [![Colab Badge](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/) |-->

## Examples of {{our}} for Image Classification

{% set mnist_example_files %}<a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/mnist_example.ipynb"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/ipynb_icon.png" alt="Jupyter Logo" style="height: 2em; vertical-align: middle; margin-right: 10px;"></a> <a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/configs/mnist_configs.yaml"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/yaml_icon.png" alt="Yaml Logo" style="height: 2em; vertical-align: middle; margin-right: 4px;"></a> <a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/mnist_example.py"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/python_icon.svg" alt="Python Logo" style="height: 2em; vertical-align: middle; margin-right: 10px;"></a>{% endset %}
{% set mnist_dual_lphm_example_files %}<a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/mnist_dual_lphm_example.ipynb"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/ipynb_icon.png" alt="Jupyter Logo" style="height: 2em; vertical-align: middle; margin-right: 10px;"></a> <a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/configs/mnist_dual_lphm_configs.yaml"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/yaml_icon.png" alt="Yaml Logo" style="height: 2em; vertical-align: middle; margin-right: 4px;"></a> <a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/mnist_dual_lphm_example.py"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/python_icon.svg" alt="Python Logo" style="height: 2em; vertical-align: middle; margin-right: 10px;"></a>{% endset %}
{% set cifar10_example_files %}<a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/cifar10_example.ipynb"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/ipynb_icon.png" alt="Jupyter Logo" style="height: 2em; vertical-align: middle; margin-right: 10px;"></a> <a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/configs/cifar10_configs.yaml"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/yaml_icon.png" alt="Yaml Logo" style="height: 2em; vertical-align: middle; margin-right: 4px;"></a> <a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/cifar10_example.py"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/python_icon.svg" alt="Python Logo" style="height: 2em; vertical-align: middle; margin-right: 10px;"></a>{% endset %}


|               Example ID                |                   Example Title                    | Released Date  |           Related Files           |
|:---------------------------------------:|:--------------------------------------------------:|:--------------:|:---------------------------------:|
|      [Example 4](./image/mnist.md)      | MNIST Classification with Identity Reconciliation  |  July 8, 2024  |      {{mnist_example_files}}      |
| [Example 5](./image/mnist_dual_lphm.md) | MNIST Classification with Dual LPHM Reconciliation |  July 8, 2024  | {{mnist_dual_lphm_example_files}} |
|     [Example 6](./image/cifar10.md)     |         CIFAR10 Object Detection in Images         |  July 8, 2024  |     {{cifar10_example_files}}     |


## Examples of {{our}} for Text Classification

{% set kan_failure_example_files %}<a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/kan_failure_example.ipynb"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/ipynb_icon.png" alt="Jupyter Logo" style="height: 2em; vertical-align: middle; margin-right: 10px;"></a> <a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/configs/kan_failure_configs.yaml"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/yaml_icon.png" alt="Yaml Logo" style="height: 2em; vertical-align: middle; margin-right: 4px;"></a> <a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/kan_failure_example.py"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/python_icon.svg" alt="Python Logo" style="height: 2em; vertical-align: middle; margin-right: 10px;"></a>{% endset %}
{% set imdb_example_files %}<a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/imdb_example.ipynb"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/ipynb_icon.png" alt="Jupyter Logo" style="height: 2em; vertical-align: middle; margin-right: 10px;"></a> <a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/configs/imdb_configs.yaml"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/yaml_icon.png" alt="Yaml Logo" style="height: 2em; vertical-align: middle; margin-right: 4px;"></a> <a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/imdb_example.py"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/python_icon.svg" alt="Python Logo" style="height: 2em; vertical-align: middle; margin-right: 10px;"></a>{% endset %}
{% set sst2_example_files %}<a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/sst2_example.ipynb"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/ipynb_icon.png" alt="Jupyter Logo" style="height: 2em; vertical-align: middle; margin-right: 10px;"></a> <a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/configs/sst2_configs.yaml"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/yaml_icon.png" alt="Yaml Logo" style="height: 2em; vertical-align: middle; margin-right: 4px;"></a> <a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/sst2_example.py"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/python_icon.svg" alt="Python Logo" style="height: 2em; vertical-align: middle; margin-right: 10px;"></a>{% endset %}
{% set agnews_example_files %}<a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/agnews_example.ipynb"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/ipynb_icon.png" alt="Jupyter Logo" style="height: 2em; vertical-align: middle; margin-right: 10px;"></a> <a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/configs/agnews_configs.yaml"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/yaml_icon.png" alt="Yaml Logo" style="height: 2em; vertical-align: middle; margin-right: 4px;"></a> <a href="https://github.com/jwzhanggy/tinyBIG/blob/main/docs/notes/agnews_example.py"><img src="https://raw.githubusercontent.com/jwzhanggy/tinyBIG/main/docs/assets/img/python_icon.svg" alt="Python Logo" style="height: 2em; vertical-align: middle; margin-right: 10px;"></a>{% endset %}


|          Example ID           |          Example Title          | Released Date  |         Related Files         |
|:-----------------------------:|:-------------------------------:|:--------------:|:-----------------------------:|
|  [Example 7](./text/kan.md)   |  Failure of KAN on Sparse Data  |  July 9, 2024  | {{kan_failure_example_files}} |
|  [Example 8](./text/imdb.md)  |   IMDB Review Classification    |  July 9, 2024  |    {{imdb_example_files}}     |
| [Example 9](./text/agnews.md) |   AGNews Topic Classification   |  July 9, 2024  |    {{sst2_example_files}}     |
| [Example 10](./text/sst2.md)  | SST-2 Sentiment Classification  |  July 9, 2024  |   {{agnews_example_files}}    |

## Examples of {{our}} for Probabilistic Relationship Inference

|   Example ID   |           Example Title           | Released Date |   Related Files    |
|:--------------:|:---------------------------------:|:------------:|:---------------:|
| [Example 9]()  |      Iris Species Inference       |     TBD      | To Be Provided  |
| [Example 10]() |   Diabetes Diagnosis Inference    |     TBD      | To Be Provided  |
| [Example 11]() | Banknote Authentication Inference |     TBD      | To Be Provided  |


